﻿using ExternalServiceV3.Models;
using Microsoft.AspNetCore.Mvc;
using System.Text.Json;
using System.Threading.Tasks;
using Microsoft.Extensions.Logging;

namespace ExternalServiceV3.Controllers
{
   [Route("api/3.5/authenticate/")]
   [ApiController]
   public class AuthenticationController : ControllerBase
   {
      private readonly ILogger<AuthenticationController> _logger;
      private readonly DataAccess _dataAccess;

      public AuthenticationController(ILogger<AuthenticationController> logger, DataAccess dataAccess)
      {
         _logger = logger;
         _dataAccess = dataAccess;
      }

      // POST api/authenticate/
      [HttpPost]
      [HeaderAuthorization]
      public async Task<ActionResult<AuthenticationResult>> Post(AuthenticationRequest request)
      {
         _logger.LogDebug("POST AUTHENTICATION ENDPOINT");
         _logger.LogDebug("Payload received: {Request}", SerializeMaskingPassword(request));

         AuthenticationResult results;
         switch (request.Type)
         {
            case "HashedUserCredentials": //Username, Password, HashingKey, HashingVersion, CaseSensitivePassword //This is an Request Type for an old obsoleted scenario.
               results = Helpers.GenerateUnsuccessfulAuthenticationResult("HashedUserCredentials request is not supported by this implementation of External Service V3");
               break;
            default: 
               results = await ProcessRequestAsync(request);
               break;
         }

         _logger.LogDebug("Results: {Results}", JsonSerializer.Serialize(results));
         _logger.LogDebug("Post Authentication ends");

         return results;
      }

      [HttpGet]
      public ActionResult<string> Get()
      {
         _logger.LogError("GET AUTHENTICATION ENDPOINT");
         return "This is an example of JSON API v3.5 it allows you to get a direct authentication and authorization within one request.";
      }
      
      private async Task<AuthenticationResult> ProcessRequestAsync(AuthenticationRequest request)
      {
         //There is an username and password, we need to check username/password
         if (request.Username != null && request.Password != null)
         {
            if (request.Document != null) //Case 1: we have a document
            {
               return await _dataAccess.GetAuthenticationAsync(request.Username, request.Password,
                  request.CaseSensitivePassword, request.Document);
            }
            else //Case 2: document is null
            {
               var reader = await _dataAccess.GetReaderByUsernameAndPasswordAsync(request.Username, request.Password,
                  request.CaseSensitivePassword);
               if (reader == null)
                  return Helpers.GenerateUnsuccessfulAuthenticationResult(null);

               return Helpers.GenerateSuccessfulAuthenticationResult(reader);
            }
         }

         //There is an username, but there is no password. we need to check username. The user might have already signed in previously, or could be signing in with web viewer single sign on with Oauth or SAML
         if (request.Username != null && request.Password == null)
         {
            if (request.Document != null) //Case 1: we have a document
            {
               return await _dataAccess.GetAuthenticationAsync(request.Username, request.Document);
            }
            else //Case 2: document is null.
            {
               var reader = await _dataAccess.GetReaderByUsernameAsync(request.Username);
               if (reader == null)
                  return Helpers.GenerateUnsuccessfulAuthenticationResult(
                     "Your account could not be found in the system. Make sure you are entering the correct details (3yq)");

               return Helpers.GenerateSuccessfulAuthenticationResult(reader);
            }
         }
         
         //We know there is no username and no username/password. we need to check the token as the user could be signing in using web viewer single sign on with query parameter or cookie
         if (request.Token != null)
         {
            if (request.Document != null) //Case 1: we have a document
            {
               return await _dataAccess.GetAuthenticationByTokenAsync(request.Token, request.Document);
            }
            else //Case 2: document is null.
            {
               var reader = await _dataAccess.GetReaderByTokenAsync(request.Token);
               if (reader != null)
               {
                  return Helpers.GenerateSuccessfulAuthenticationResult(reader);
               }
            }
         }
         
         //we know there is no username/password/token. we need to check the id as the user could opening a document using pdf sso lite
         if (request.Id != null)
         {
            if (request.Document != null) //Case 1: we have a document
            {
               return await _dataAccess.GetAuthenticationByIdAsync(request.Id, request.Document);
            }
            else //Case 2: document is null.
            {
               var reader = await _dataAccess.GetReaderByIdAsync(request.Id);
               if (reader != null)
               {
                  return Helpers.GenerateSuccessfulAuthenticationResult(reader);
               }
            }
         }

         //All the condition statements above are written so that they will return if execution falls into them. If it got here, it means the request didn't match any of the scenario above.
         _logger.LogDebug($"Unable to process request for {request.Type}, {JsonSerializer.Serialize(request)}");
         return Helpers.GenerateUnsuccessfulAuthenticationResult($"Unable to process request for {request.Type}");
      }

      private static string SerializeMaskingPassword(AuthenticationRequest request)
      {
         if (string.IsNullOrEmpty(request?.Password))
            return JsonSerializer.Serialize(request);

         AuthenticationRequest requestClone = JsonSerializer.Deserialize<AuthenticationRequest>(JsonSerializer.Serialize(request));
         requestClone.Password = new string('*', requestClone.Password.Length);

         return JsonSerializer.Serialize(requestClone);
      }
   }
}
