using System;
using System.Text;
using Microsoft.AspNetCore.Http;
using Microsoft.AspNetCore.Http.Extensions;
using Microsoft.AspNetCore.Mvc;
using Microsoft.AspNetCore.Mvc.Filters;
using Microsoft.Extensions.Configuration;
using Microsoft.Extensions.Logging;

namespace ExternalServiceV3;

[AttributeUsage(AttributeTargets.Method)]
public class HeaderAuthorizationAttribute : TypeFilterAttribute
{
   public HeaderAuthorizationAttribute() : base(typeof(HeaderAuthorizationFilter))
   {
   }

   private class HeaderAuthorizationFilter : IActionFilter
   {
      private readonly bool _apiHeaderValidationEnabled;
      private readonly string _apiHeaderValidationHeaderKey;
      private readonly string _apiHeaderValidationHeaderValue;
      private readonly ILogger<HeaderAuthorizationFilter> _logger;

      public HeaderAuthorizationFilter(ILogger<HeaderAuthorizationFilter> logger, IConfiguration configuration)
      {
         _apiHeaderValidationEnabled = bool.Parse(configuration["ApiHeaderValidation:Enabled"]);
         _apiHeaderValidationHeaderKey = configuration["ApiHeaderValidation:HeaderKey"];
         _apiHeaderValidationHeaderValue = configuration["ApiHeaderValidation:HeaderValue"];
         _logger = logger;
      }

      public void OnActionExecuting(ActionExecutingContext context)
      {
         if (_apiHeaderValidationEnabled)
         {
            if (string.IsNullOrWhiteSpace(_apiHeaderValidationHeaderKey) || string.IsNullOrWhiteSpace(_apiHeaderValidationHeaderValue))
            {
               context.Result = new UnauthorizedResult();
               return;
            }

            var headers = context.HttpContext.Request.Headers;
            if (!headers.ContainsKey(_apiHeaderValidationHeaderKey)
                || headers[_apiHeaderValidationHeaderKey] != _apiHeaderValidationHeaderValue)
            {
               _logger.LogDebug(
                  $"HeaderAuthorizationFilter: Rejected call to {context.HttpContext.Request.GetDisplayUrl()} due to missing header.");
               _logger.LogDebug($"Source IP: {context.HttpContext.Connection.RemoteIpAddress}");
               _logger.LogDebug($"Headers: {ConvertHeadersToString(headers)}");
               context.Result = new UnauthorizedResult();
            }
         }
      }

      public void OnActionExecuted(ActionExecutedContext context)
      {
         // Do nothing after action execution
      }
      
      static string ConvertHeadersToString(IHeaderDictionary headers)
      {
         StringBuilder sb = new StringBuilder();

         foreach (var header in headers)
         {
            sb.Append(header.Key);
            sb.Append(": ");
            sb.AppendLine(string.Join(", ", header.Value));
         }

         return sb.ToString();
      }
   }
}