﻿using System;
using System.Collections.Generic;

// ****************************
// NOTE: The XML documentation here is copied into ReaderPolicy as well as Server API's UniqueDocAccessOverride. When making changes, make sure you update all.
// ****************************

namespace ExternalServiceV3.Models
{
   /// <summary>
   /// Access (DRM) Policy is at the root of controlling end users' access to protected content. 
   /// <b>IMPORTANT NOTE</b>: Policies can exist at multiple levels (global, content). When multiple policies apply, Vitrium uses 
   /// a most lenient combination of applicable Access Policy settings in an attempt to grant access to protected content. When a particular field validation
   /// fails for one policy, it may pass for another and grant access to content. It is important to be aware of this concept when combining Access Policies
   /// at different levels as it may lead to authorization results which are difficult to trace. It is advisable to only use AccessPolicies at a single chosen
   /// level and set irrelevant fields to Unlimited (generally null but documented for each setting).
   /// </summary>
   public class AccessPolicy
   {
      public static DateTime DateNotSet = new DateTime(1900, 1, 1);
      public const int NotSet = -1;

      /// <summary>
      /// Optional limit on the number of separate devices on which a content can be opened, combining Web Viewer and Adobe content access. If omitted or set to null, the value is Unlimited. 
      /// Value of -1 is equivalent to Not Set, which fails validation and instead use PdfLimit (for downloaded PDF content opened in Adobe) 
      /// or BrowserLimit (for content accessed through Web Viewer). Note that Web Viewer and Adobe access is tracked separately, so opening the same content in Web Viewer 
      /// and Adobe (as downloaded PDF) will add 2 to this combined ComputersMax device access count. Note also that this limit is closely related to PdfLimit and BrowserLimit, and is checked
      /// first when authorizing content access. A large value in this field may cause an access policy to be valid even if individual PdfLimit/BrowserLimit would be exceeded.
      /// It is therefore recommended that if PdfLimit and/or BrowserLimit are used, ComputersMax be set to -1. Conversely, if using ComputersMax, it is recommended that PdfLimit and BrowserLimit
      /// both be set to -1 to prevent an Access Policy from authorizing access through those two fields if ComputersMax is exceeded.
      /// </summary>
      public int? ComputersMax { get; set; }

      /// <summary>
      /// Optional limit on the number of separate devices on which downloaded PDF content can be opened in Adobe. If omitted or set to null, the value is Unlimited. 
      /// Value of -1 is equivalent to Not Set and will fail validation. If using this setting, 
      /// it is recommended that ComputersMax be set to -1 (see ComputersMax field documentation).
      /// </summary>
      public int? PdfLimit { get; set; }

      /// <summary>
      /// Optional limit on the number of separate devices on which content can be opened in Web Viewer. If omitted or set to null, the value is Unlimited. 
      /// Value of -1 is equivalent to Not Set and will fail validation. If using this setting, 
      /// it is recommended that ComputersMax be set to -1 (see ComputersMax field documentation).
      /// </summary>
      public int? BrowserLimit { get; set; }

      /// <summary>
      /// Optional limit on the number of different IP Addresses from which content can be opened. Applies to both Web Viewer and downloaded PDF content opened in Adobe.
      /// If omitted or set to null, the value is Unlimited. Value of -1 is equivalent to Not Set and will fail validation for this Access Policy. 
      /// </summary>
      public int? IpAddressesMax { get; set; }

      /// <summary>
      /// Optional list of individual IP addresses or IP address range(s) to be white-listed. Works in conjuction with the IpAddressesMax setting. IP addresses included in this setting 
      /// do not count towards the IpAddressesMax limit. Example: "127.0.0.1, 127.0.0.5, 127.0.0.10-127.0.0.19"
      /// </summary>
      public string IgnoredIpAddresses { get; set; }

      /// <summary>
      /// Optional date and time on which access to content expires. This date is converted internally to UTC using the Time Zone under Account Settings to preserve the universal date and time.
      /// If omitted or set to null, the value is Unlimited. Value of 01/01/1900 is used internally to indicate Not Set, but you can use any date in the past to fail validation for this Access Policy.
      /// Note that this setting complements RelativeExpiryInDays; if both are specified, the <i>earlier</i> date will determine the content access expiry.
      /// </summary>
      public DateTime? Expiry { get; set; }

      /// <summary>
      /// Optional number of days for which the content will be accessible after it is first unlocked. If omitted or set to null, the value is Unlimited. 
      /// Value of -1 is equivalent to Not Set and will fail validation for this Access Policy.
      /// Note that this setting complements Expiry; if both are specified, the <i>earlier</i> date will determine the content access expiry.
      /// </summary>
      public long? RelativeExpiryInDays { get; set; }

      /// <summary>
      /// Optional number of days that an unlocked document will be available offline. Internet connection is still required for an initial unlock, after which this value will be
      /// stored in a cookie for subsequent offline access. Applies to both downloaded PDFs and Web Viewer content Saved to Browser. If omitted or set to null, the value is Unlimited. 
      /// Value of -1 is equivalent to Not Set and will fail offline validation for this Access Policy.
      /// Note that this setting is related to and can be overridden by Expiry and RelativeExpiryInDays if those expire earlier. In other words, Vitrium internally updates this field to 
      /// be the earliest of OfflineDurationInDays, RelativeExpiryInDays, and Expiry. The inverse is not true: RelativeExpiryInDays and Expiry are independent and not affected by this or
      /// any other setting.
      /// </summary>
      public long? OfflineDurationinDays { get; set; }

      /// <summary>
      /// Start date on which this Access Policy will come into effect. This can be useful in subscription scenarios when content access should be granted from a given start date.
      /// </summary>
      public DateTime StartDate { get; set; }

      /// <summary>
      /// Optional limit on the number of separate documents the end user can access using this Access Policy. If omitted or set to null, the value is Unlimited. 
      /// Value of -1 is equivalent to Not Set and will fail offline validation for this Access Policy.
      /// </summary>
      public int? DocumentLimit { get; set; }

      /// <summary>
      /// Optional limit on the number of times a particular document can accessed using this Access Policy. If omitted or set to null, the value is Unlimited. 
      /// Value of -1 is equivalent to Not Set and will fail offline validation for this Access Policy.
      /// </summary>
      public int? OpenLimit { get; set; }

      /// <summary>
      /// Optional limit on the number of times the Download To Print function can be used in the Web Viewer for a particular document. If omitted or set to null, the value is Unlimited. 
      /// Value of -1 is equivalent to Not Set and will fail offline validation for this Access Policy.
      /// </summary>
      public int? PrintLimit { get; set; }

      /// <summary>
      /// Optional limit on the number of times the a particular document can be printed from the Web Viewer. If omitted or set to null, the value is Unlimited. 
      /// Value of -1 is equivalent to Not Set and will fail offline validation for this Access Policy. Note that these is no way to limit the number of times downloaded
      /// content can be opened in Adobe, but a Content Setting Policy can be used to disable Adobe printing.
      /// </summary>
      public int? WebPrintLimit { get; set; }

    
      /// <summary>
      /// Optional boolean that allows you to override the DisableAnnotations setting from Content Policy for Web Viewer ONLY. This has no effect on annotations for downloaded 
      /// PDF content opened in Adobe, which is controlled exclusively by Content Policy (documented under Server API Documentation).
      /// Implementation Note: This is equivalent to WebViewerDocPolicy's AllowAnnotations field, but has a lower precedence. If both AllowAnnotations fields are provided, 
      /// WebViewerDocPolicy's AllowAnnotations will prevail.
      /// </summary>
      public bool? AnnotationsAllowed { get; set; }

      /// <summary>
      /// Optional WebViewerDocPolicy object allowing you to override Content Policy settings as they apply to Web Viewer ONLY. This setting has no effect on downloaded PDF content opened in Adobe.
      /// </summary>
      public WebViewerDocPolicy WebViewerDocPolicyOverride { get; set; }

      /// <summary>
      /// Optional boolean that allows you to set this Access Policy as the Global Access Policy, which is examined for all content access. Value of null or false has no effect; true will set this Access
      /// Policy to be the Global Access Policy. Note the comments in the class documentation about the risks of multiple Access Policy Levels. Note also that there is no API to delete a Global Access Policy
      /// setting, though the Admin UI provides this functionality.
      /// </summary>
      public bool? SetAsGlobal { get; set; }

      /// <summary>
      /// Optional string which can store any value for your internal use.
      /// </summary>
      public string CustomField { get; set; }

      public bool? AllowDownloadSourceFile { get; set; }

      public List<string> LocationRestrictions { get; set; }

      public List<string> LocationPermits { get; set; }
      
      public bool IsWebAccessibilityEnabled { get; set; }
      
      public bool AllowProtectedDownload { get; set; }
      
      public bool AllowAnnotationViewing { get; set; }
      
      /// <summary>
      /// Optional limit on the number of concurrent logins to the same document.
      /// If omitted or set to null, the value is Unlimited. Value of -1 is equivalent to Not Set and will fail validation.
      /// </summary>
      public int? ConcurrentUsersLimit { get; set; }
   }

   public class WebViewerDocPolicy
   {      
      public bool? AllowCopy { get; set; }
      public bool? AllowPrint { get; set; }
      public bool? AllowWebPrint { get; set; }
      public bool? DisableBookmarks { get; set; }
      public bool? DisableSearch { get; set; }
      public bool? DisablePageRangePrint { get; set; }
      public bool? DisableRotations { get; set; }   
      public bool DisableFreeHandAnnotations { get; set; }
      public bool DisableTextAnnotations { get; set; }
      public bool DisableTextDecorationAnnotations { get; set; }
      public bool DisableNoteAnnotations { get; set; }
      public bool DisableShapeAnnotations { get; set; }
      public bool DisableMeasureTools { get; set; }
   }
}