using System.Threading.Tasks;
using ExternalServiceV3;
using ExternalServiceV3.Controllers;
using ExternalServiceV3.Models;
using Microsoft.Extensions.DependencyInjection;
using Microsoft.Extensions.Logging;
using Microsoft.VisualStudio.TestTools.UnitTesting;

namespace FunctionalTests
{
   [TestClass]
   public class AuthenticationControllerTests: BaseTest
   {
      [TestMethod]
      public async Task UserCredentials_Test()
      {
         var controller = new AuthenticationController(ServiceProvider.GetService<ILogger<AuthenticationController>>(), ServiceProvider.GetService<DataAccess>());
         var results = await controller.Post(new AuthenticationRequest()
         {
            Type = "UserCredentials",
            Username = "user8@domain.com",
            Password = "test123",
            CaseSensitivePassword = false,
            Document = new DocumentContext() { ExternalKey = "ValidExternalKey" },
         });

         Assert.IsTrue(results.Value.Succeed);
         Assert.AreEqual("user8@domain.com", results.Value.Username);
         Assert.AreEqual("8", results.Value.UserId);
      }
      
      [TestMethod]
      public async Task UserCredentials_CorrectUsername_WrongPassword_Credential_Test()
      {
         var controller = new AuthenticationController(ServiceProvider.GetService<ILogger<AuthenticationController>>(), ServiceProvider.GetService<DataAccess>());
         var results = await controller.Post(new AuthenticationRequest()
         {
            Type = "UserCredentials",
            Username = "user8@domain.com",
            Password = "wrong_passsword",
            CaseSensitivePassword = false,
            Document = new DocumentContext() { ExternalKey = "not_important" },
         });

         Assert.IsFalse(results.Value.Succeed);
         Assert.AreEqual("user8@domain.com", results.Value.Username);
         Assert.IsTrue(results.Value.Message.Contains("bw5"));
      }
      
      [TestMethod]
      public async Task UserCredentials_WrongUsername_Test()
      {
         var controller = new AuthenticationController(ServiceProvider.GetService<ILogger<AuthenticationController>>(), ServiceProvider.GetService<DataAccess>());
         var results = await controller.Post(new AuthenticationRequest()
         {
            Type = "UserCredentials",
            Username = "wrong_username",
            Password = "wrong_passsword",
            CaseSensitivePassword = false,
            Document = new DocumentContext() { ExternalKey = "not_important" },
         });

         Assert.IsFalse(results.Value.Succeed);
         Assert.AreEqual("wrong_username", results.Value.Username);
         Assert.IsTrue(results.Value.Message.Contains("3yq"));
      }

      [TestMethod]
      public async Task UserCredentials_NoPermission_Failed_Test()
      {
         var controller = new AuthenticationController(ServiceProvider.GetService<ILogger<AuthenticationController>>(), ServiceProvider.GetService<DataAccess>());
         var results = await controller.Post(new AuthenticationRequest()
         {
            Type = "UserCredentials",
            Username = "user8@domain.com",
            Password = "test123",
            CaseSensitivePassword = false,
            Document = new DocumentContext() { ExternalKey = "InvalidExternalKey" },
         });

         Assert.IsFalse(results.Value.Succeed);
         Assert.AreEqual("user8@domain.com", results.Value.Username);
         Assert.AreEqual("8", results.Value.UserId);
      }

      [TestMethod]
      public async Task UniqueDocCopyIdTokens_Test()
      {
         var controller = new AuthenticationController(ServiceProvider.GetService<ILogger<AuthenticationController>>(), ServiceProvider.GetService<DataAccess>());
         var results = await controller.Post(new AuthenticationRequest()
         {
            Type = "UniqueDocCopyIdToken",
            Username = "user8@domain.com",
            Document = new DocumentContext() { ExternalKey = "ValidExternalKey" },
         });

         Assert.IsTrue(results.Value.Succeed);
         Assert.AreEqual("user8@domain.com", results.Value.Username);
         Assert.AreEqual("8", results.Value.UserId);
      }

      [TestMethod]
      public async Task SsoLiteToken_Test()
      {
         var controller = new AuthenticationController(ServiceProvider.GetService<ILogger<AuthenticationController>>(), ServiceProvider.GetService<DataAccess>());
         var results = await controller.Post(new AuthenticationRequest()
         {
            Type = "SsoLiteToken",
            Id = "8",
            Document = new DocumentContext() { ExternalKey = "ValidExternalKey" },
         });

         Assert.IsTrue(results.Value.Succeed);
         Assert.AreEqual("user8@domain.com", results.Value.Username);
         Assert.AreEqual("8", results.Value.UserId);
      }

      [TestMethod]
      public async Task WebViewerSso_Test()
      {
         var controller = new AuthenticationController(ServiceProvider.GetService<ILogger<AuthenticationController>>(), ServiceProvider.GetService<DataAccess>());
         var results = await controller.Post(new AuthenticationRequest()
         {
            Type = "WebViewerSso",
            Token = "3BBD5CC2-954A-40A1-8653-C14D8B4B4824",
            Document = new DocumentContext() { ExternalKey = "ValidExternalKey" }
         });

         Assert.IsTrue(results.Value.Succeed);
         Assert.AreEqual("user8@domain.com", results.Value.Username);
         Assert.AreEqual("8", results.Value.UserId);
      }


      [TestMethod]
      public async Task PhoneUnlockToken_Test()
      {
         var controller = new AuthenticationController(ServiceProvider.GetService<ILogger<AuthenticationController>>(), ServiceProvider.GetService<DataAccess>());
         var results = await controller.Post(new AuthenticationRequest()
         {
            Type = "PhoneUnlockToken",
            Username = "user8@domain.com",
            Document = new DocumentContext() { ExternalKey = "ValidExternalKey" }
         });

         Assert.IsTrue(results.Value.Succeed);
         Assert.AreEqual("user8@domain.com", results.Value.Username);
         Assert.AreEqual("8", results.Value.UserId);
      }

      [TestMethod]
      public async Task PrintMeteringUsernameToken_Test()
      {
         var controller = new AuthenticationController(ServiceProvider.GetService<ILogger<AuthenticationController>>(), ServiceProvider.GetService<DataAccess>());
         var results = await controller.Post(new AuthenticationRequest()
         {
            Type = "PrintMeteringUsernameToken",
            Username = "user8@domain.com",
            Document = new DocumentContext() { ExternalKey = "ValidExternalKey" }
         });

         Assert.IsTrue(results.Value.Succeed);
         Assert.AreEqual("user8@domain.com", results.Value.Username);
         Assert.AreEqual("8", results.Value.UserId);
      }

      [TestMethod]
      public async Task WebViewerSessionTokenVerification_Test()
      {
         var controller = new AuthenticationController(ServiceProvider.GetService<ILogger<AuthenticationController>>(), ServiceProvider.GetService<DataAccess>());
         var results = await controller.Post(new AuthenticationRequest()
         {
            Type = "WebViewerSessionTokenVerification",
            Username = "user8@domain.com",
            Document = new DocumentContext() { ExternalKey = "ValidExternalKey" }
         });

         Assert.IsTrue(results.Value.Succeed);
         Assert.AreEqual("user8@domain.com", results.Value.Username);
         Assert.AreEqual("8", results.Value.UserId);
      }
   }
}